@@ -39,15 +39,16 @@ char copyright[] =
  * From: @(#)telnetd.c	5.48 (Berkeley) 3/1/91
  */
 char telnetd_rcsid[] = 
-  "$Id: telnetd.c,v 1.16 1999/08/19 01:05:40 dholland Exp $";
+  "$Id: telnetd.c,v 1.23 1999/12/14 00:43:31 dholland Exp $";
 
 #include "../version.h"
 
 #include <netdb.h>
 #include <termcap.h>
 #include <netinet/in.h>
-#include <netinet/ip.h>
+/* #include <netinet/ip.h> */ /* Don't think this is used at all here */
 #include <arpa/inet.h>
+#include <assert.h>
 #include "telnetd.h"
 #include "pathnames.h"
 #include "setproctitle.h"
@@ -63,7 +64,7 @@ int	require_SecurID = 0;
 #endif
 
 static void doit(struct sockaddr_in *who);
-static int terminaltypeok(char *s);
+static int terminaltypeok(const char *s);
 
 /*
  * I/O data buffers,
@@ -86,7 +87,7 @@ main(int argc, char *argv[], char *env[])
 {
 	struct sockaddr_in from;
 	int on = 1;
-	size_t fromlen;
+	socklen_t fromlen;
 	register int ch;
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
@@ -195,7 +196,8 @@ main(int argc, char *argv[], char *env[])
 #endif	/* LINEMODE */
 
 		case 'L':
-			loginprg = optarg;
+			loginprg = strdup(optarg);
+			/* XXX what if strdup fails? */
 			break;
 
 		case 'n':
@@ -243,25 +245,29 @@ main(int argc, char *argv[], char *env[])
 
 	if (debug) {
 	    int s, ns;
-	    size_t foo;
+	    socklen_t foo;
 	    struct servent *sp;
-	    static struct sockaddr_in sn = { AF_INET };
+	    struct sockaddr_in sn;
+
+	    memset(&sn, 0, sizeof(sn));
+	    sn.sin_family = AF_INET;
 
 	    if (argc > 1) {
 		usage();
-		/* NOT REACHED */
+		/* NOTREACHED */
 	    } else if (argc == 1) {
 		    if ((sp = getservbyname(*argv, "tcp"))!=NULL) {
 			sn.sin_port = sp->s_port;
 		    } 
 		    else {
-			sn.sin_port = atoi(*argv);
-			if ((int)sn.sin_port <= 0) {
-			    fprintf(stderr, "telnetd: %s: bad port #\n", *argv);
+			int pt = atoi(*argv);
+			if (pt <= 0) {
+			    fprintf(stderr, "telnetd: %s: bad port number\n",
+				    *argv);
 			    usage();
-			    /* NOT REACHED */
+			    /* NOTREACHED */
 			}
-			sn.sin_port = htons((u_short)sn.sin_port);
+			sn.sin_port = htons(pt);
 		   }
 	    } else {
 		sp = getservbyname("telnet", "tcp");
@@ -372,7 +378,6 @@ usage(void)
  *	Ask the other end to send along its terminal type and speed.
  * Output is the variable terminaltype filled in.
  */
-static char ttytype_sbbuf[] = { IAC, SB, TELOPT_TTYPE, TELQUAL_SEND, IAC, SE };
 
 static void _gettermname(void);
 
@@ -423,27 +428,20 @@ getterminaltype(char *name)
     }
 #endif
     if (his_state_is_will(TELOPT_TSPEED)) {
-	static char sbbuf[] = { IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
-
-	bcopy(sbbuf, nfrontp, sizeof sbbuf);
-	nfrontp += sizeof sbbuf;
+	netoprintf("%c%c%c%c%c%c", 
+		   IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE);
     }
     if (his_state_is_will(TELOPT_XDISPLOC)) {
-	static char sbbuf[] = { IAC, SB, TELOPT_XDISPLOC, TELQUAL_SEND, IAC, SE };
-
-	bcopy(sbbuf, nfrontp, sizeof sbbuf);
-	nfrontp += sizeof sbbuf;
+	netoprintf("%c%c%c%c%c%c", 
+		   IAC, SB, TELOPT_XDISPLOC, TELQUAL_SEND, IAC, SE);
     }
     if (his_state_is_will(TELOPT_ENVIRON)) {
-	static char sbbuf[] = { IAC, SB, TELOPT_ENVIRON, TELQUAL_SEND, IAC, SE };
-
-	bcopy(sbbuf, nfrontp, sizeof sbbuf);
-	nfrontp += sizeof sbbuf;
+	netoprintf("%c%c%c%c%c%c", 
+		   IAC, SB, TELOPT_ENVIRON, TELQUAL_SEND, IAC, SE);
     }
     if (his_state_is_will(TELOPT_TTYPE)) {
-
-	bcopy(ttytype_sbbuf, nfrontp, sizeof ttytype_sbbuf);
-	nfrontp += sizeof ttytype_sbbuf;
+       netoprintf("%c%c%c%c%c%c", 
+		  IAC, SB, TELOPT_TTYPE, TELQUAL_SEND, IAC, SE);
     }
     if (his_state_is_will(TELOPT_TSPEED)) {
 	while (sequenceIs(tspeedsubopt, baseline))
@@ -532,17 +530,17 @@ _gettermname(void)
      */
     if (his_state_is_wont(TELOPT_TTYPE))
 	return;
+
     settimer(baseline);
-    bcopy(ttytype_sbbuf, nfrontp, sizeof ttytype_sbbuf);
-    nfrontp += sizeof ttytype_sbbuf;
+    netoprintf("%c%c%c%c%c%c", IAC, SB, TELOPT_TTYPE, TELQUAL_SEND, IAC, SE);
     while (sequenceIs(ttypesubopt, baseline))
 	ttloop();
 }
 
 static int
-terminaltypeok(char *s)
+terminaltypeok(const char *s)
 {
-    char buf[2048];
+    /* char buf[2048]; */
 
     if (terminaltype == NULL)
 	return(1);
@@ -555,6 +553,22 @@ terminaltypeok(char *s)
 	return 0;
     }
 
+    /*
+     * If it's absurdly long, accept it without asking termcap.
+     *
+     * This means that it won't get seen again until after login,
+     * at which point exploiting buffer problems in termcap doesn't
+     * gain one anything.
+     *
+     * It's possible this limit ought to be raised to 128, but nothing
+     * in my termcap is more than 64, 64 is _plenty_ for most, and while
+     * buffers aren't likely to be smaller than 64, they might be 80 and
+     * thus less than 128.
+     */
+    if (strlen(s) > 63) {
+       return 0;
+    }
+
     /*
      * tgetent() will return 1 if the type is known, and
      * 0 if it is not known.  If it returns -1, it couldn't
@@ -562,8 +576,17 @@ terminaltypeok(char *s)
      * it won't help to say we failed, because we won't be
      * able to verify anything else.  So, we treat -1 like 1.
      */
-    if (tgetent(buf, s) == 0)
-	return(0);
+
+    /*
+     * Don't do this - tgetent is not really trustworthy. Assume
+     * the terminal type is one we know; terminal types are pretty
+     * standard now. And if it isn't, it's unlikely we're going to
+     * know anything else the remote telnet might send as an alias
+     * for it.
+     *
+     * if (tgetent(buf, s) == 0)
+     *    return(0);
+     */
     return(1);
 }
 
@@ -754,9 +777,7 @@ void telnet(int f, int p)
      * mode, which we do not want.
      */
     if (his_want_state_is_will(TELOPT_ECHO)) {
-	DIAG(TD_OPTIONS,
-	     {sprintf(nfrontp, "td: simulating recv\r\n");
-	     nfrontp += strlen(nfrontp);});
+	DIAG(TD_OPTIONS, netoprintf("td: simulating recv\r\n"););
 	willoption(TELOPT_ECHO);
     }
     
@@ -849,9 +870,7 @@ void telnet(int f, int p)
     localstat();
 #endif	/* LINEMODE */
 
-    DIAG(TD_REPORT,
-	 {sprintf(nfrontp, "td: Entering processing loop\r\n");
-	 nfrontp += strlen(nfrontp);});
+    DIAG(TD_REPORT, netoprintf("td: Entering processing loop\r\n"););
     
     for (;;) {
 	fd_set ibits, obits, xbits;
@@ -978,8 +997,7 @@ void telnet(int f, int p)
 		netip = netibuf;
 	    }
 	    DIAG((TD_REPORT | TD_NETDATA),
-		 {sprintf(nfrontp, "td: netread %d chars\r\n", ncc);
-		 nfrontp += strlen(nfrontp);});
+		 netoprintf("td: netread %d chars\r\n", ncc););
 	    DIAG(TD_NETDATA, printdata("nd", netip, ncc));
 	}
 	
@@ -1018,19 +1036,17 @@ void telnet(int f, int p)
 		     * royally if we send them urgent
 		     * mode data.
 		     */
-		    *nfrontp++ = IAC;
-		    *nfrontp++ = DM;
+		    netoprintf("%c%c", IAC, DM);
 		    neturg = nfrontp-1; /* off by one XXX */
 #endif
 		}
 		if (his_state_is_will(TELOPT_LFLOW) &&
 		    (ptyibuf[0] &
 		     (TIOCPKT_NOSTOP|TIOCPKT_DOSTOP))) {
-		    (void) sprintf(nfrontp, "%c%c%c%c%c%c",
+			netoprintf("%c%c%c%c%c%c",
 				   IAC, SB, TELOPT_LFLOW,
 				   ptyibuf[0] & TIOCPKT_DOSTOP ? 1 : 0,
 				   IAC, SE);
-		    nfrontp += 6;
 		}
 		pcc--;
 		ptyip = ptyibuf+1;
@@ -1125,8 +1141,7 @@ void recv_ayt(void) {
 	return;
     }
 #endif
-    sprintf(nfrontp, "\r\n[%s : yes]\r\n",host_name);
-    nfrontp += strlen(nfrontp);
+    netoprintf("\r\n[%s : yes]\r\n", host_name);
 }
 
 void doeof(void) {
