@@ -2,4 +2,4 @@
  * String to embed in binaries to identify package
  */
 
-char pkg[] = "Package: netkit-telnet-0.14";
+char pkg[]="$NetKit: netkit-telnet-0.16 $";
